import threading
import os
import time

import pygst
pygst.require ("0.10")
import gst

import gobject
import glib
import gio

from mediaitem import MediaItem

# This convert method is taken from pygst tutorial
# TODO: write own
def convertGSTTimeToString (time):
    time_int = time / 1000000000
    time_str = ""
    if time_int >= 3600:
        _hours = time_int / 3600
        time_int = time_int - (_hours * 3600)
        time_str = str(_hours) + ":"
    if time_int >= 600:
        _mins = time_int / 60
        time_int = time_int - (_mins * 60)
        time_str = time_str + str(_mins) + ":"
    elif time_int >= 60:
        _mins = time_int / 60
        time_int = time_int - (_mins * 60)
        time_str = time_str + "0" + str(_mins) + ":"
    else:
        time_str = time_str + "00:"
    if time_int > 9:
        time_str = time_str + str(time_int)
    else:
        time_str = time_str + "0" + str(time_int)

    return time_str

class Media (gobject.GObject):
    __gsignals__ = { 'time': ( gobject.SIGNAL_RUN_FIRST,
                               gobject.TYPE_NONE,
                               (gobject.TYPE_STRING, gobject.TYPE_UINT64)),
                     'eos': ( gobject.SIGNAL_RUN_FIRST,
                              gobject.TYPE_NONE,
                              (gobject.TYPE_OBJECT,)),
                     'error': ( gobject.SIGNAL_RUN_FIRST,
                              gobject.TYPE_NONE,
                              (gobject.TYPE_STRING,)),
                     'tag-update': ( gobject.SIGNAL_RUN_FIRST,
                                     gobject.TYPE_NONE,
                                     (gobject.TYPE_STRING,gobject.TYPE_STRING))}
    def __init__ (self):
        self.pipeline = None
        self.uri = None
        self.timeout_id = 0
        self.position = 0
        self.volume = 1.0
        self.bin_name = 'playbin'
        self.pipeline = None
        self.siddec = None
        self.sidtune = 0
        self.sidtune_change = False

        gobject.GObject.__init__ ( self )

    def createPipeline (self):
        self.siddec = None
        self.pipeline = gst.element_factory_make (self.bin_name, 'playbin')
        if self.pipeline == None:
            self.pipeline = gst.element_factory_make ('playbin2', 'playbin')
        if self.pipeline == None:
            self.pipeline = gst.element_factory_make ('playbin', 'playbin')

        if self.pipeline != None:
            # add watch
            bus = self.pipeline.get_bus ()
            bus.add_signal_watch ()
            bus.enable_sync_message_emission ()
            bus.connect ("message", self.messageCb)
            self.pipeline.connect ("element-added", self.elementAddedCb)

    def elementAddedCb (self, bin, element):
        if element.get_name ().find ("decodebin") == 0:
            element.connect ("element-added", self.elementAddedCb)
        elif element.get_name ().find ("abin") == 0:
            element.connect ("element-added", self.elementAddedCb)
        elif element.get_name ().find ("siddec") == 0:
            self.siddec = element
            element.set_property ("tune", self.sidtune)

    def messageCb (self, bus, message):
        type = message.type
        if type == gst.MESSAGE_EOS:
            self.pipeline.set_state (gst.STATE_NULL)
            self.removeTimeout ()
            self.emit ("eos", self)
        elif type == gst.MESSAGE_ERROR:
            err, debug = message.parse_error ()
            #print "Error: %s" % err, debug

            if self.uri.find ("http://") == 0:
                self.pipeline.set_state (gst.STATE_NULL)

            self.emit ("error", err)

            if self.uri.find ("http://") != 0:
                self.pipeline.set_state (gst.STATE_NULL)
                state = self.pipeline.get_state ()
            if self.sidmode == True:
                self.pipeline = None
                uri = self.uri
                self.uri = ''
                glib.idle_add (self.idlePlay, uri)

        elif type == gst.MESSAGE_TAG:
            taglist = message.parse_tag ()
            
            try:
                artist = taglist[gst.TAG_ARTIST]
            except:
                artist = ""
            try:
                title = taglist[gst.TAG_TITLE]
            except:
                title = ""
            self.emit ("tag-update", artist, title)

    def idlePlay (self, uri):
        self.setUri (uri)
        self.play ()
        return False

    def setVolume (self, volume):
        self.volume = volume
        if self.pipeline != None:
            self.pipeline.set_property ('volume', self.volume)

    def getVolume (self):
        return self.volume

    def play (self):
        if self.pipeline == None:
            return
        if self.uri == None:
            return

        self.pipeline.set_property ('volume', self.volume)

        self.pipeline.set_state (gst.STATE_PLAYING)
        self.timeout_id = glib.timeout_add (1000, self.emitTimeCb, None)

    def stop (self):
        if self.pipeline == None:
            return
        self.pipeline.set_state (gst.STATE_NULL)
        self.removeTimeout ()

    def pause (self):
        if self.pipeline == None:
            return
        self.pipeline.set_state (gst.STATE_PAUSED)
        self.removeTimeout ()

    def seekTo (self, seek_to):
        if self.pipeline == None:
            return
        format = gst.Format (gst.FORMAT_TIME)
        flags = gst.SEEK_FLAG_ACCURATE|gst.SEEK_FLAG_FLUSH
        self.pipeline.seek_simple (format, flags, seek_to)
        self.position = seek_to

    def setUri (self, uri):
        if uri == None:
            return False
        
        if uri[0] == '/':
            nuri = 'file://'+uri
        else:
            nuri = uri

#        print 'Media.setUri', uri
        if nuri != self.uri:
            self.uri = nuri
            if self.sidtune_change == False:
                self.sidtune = 0
            self.createPipeline ()
            self.pipeline.set_property ('uri', self.uri)
        self.sidtune_change = False
        self.position = 0

        return True;

    def removeTimeout (self):
        if self.timeout_id > 0:
            if glib.source_remove (self.timeout_id) == True:
                self.timeout_id = 0

    def emitTimeCb (self, data):
        format = gst.Format (gst.FORMAT_TIME)
        position = 0
        try:
            position = self.pipeline.query_position (format)[0]
            if self.position < position:
                self.position = position
                self.emit ("time", convertGSTTimeToString (position), position)
        except:
            None
        return True;


    def setSidMode (self, mode):
        self.sidmode = mode

    def sidGetTrack (self):
        if self.siddec == None:
            return -1

        return self.siddec.get_property ("tune")

    def sidNextTrack (self):
        tune = self.sidGetTrack ()
        if tune == -1:
            return

        if tune == 100:
            tune = 0
        else:
            tune += 1

        self.sidtune = tune
        self.sidSetTune (tune)

    def sidPrevTrack (self):
        tune = self.sidGetTrack ()
        if tune == -1:
            return

        if tune == 0:
            tune = 100
        else:
            tune -= 1

        self.sidtune = tune
        self.sidSetTune (tune)

    def sidSetTune (self, tune):
        self.stop ()
        self.position = 0
        uri = self.uri
        self.sidtune_change = True
        self.uri = ""
        self.setUri (uri)
        self.play ()

########
#####
###
#
# Test media and collect tags
#
#  Not sure for all how this is done best, this seems to work so far
#
class MediaTest (threading.Thread, gobject.GObject):
    __gsignals__ = { 'supported': ( gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
                                    ('gboolean',
                                     gobject.TYPE_PYOBJECT))}
    def __init__ (self, uri):
        threading.Thread.__init__ ( self )
        gobject.GObject.__init__ ( self )

        self.emitted = False
        self.uri = None
        self.sinks = []
        self.taglist = gst.TagList ()
        self.mediacontent = u''
        self.decbin_name = 'decodebin'
        self.type = u''
        self.pipeline = None

        # does uri exist
        try:
            fp = gio.File (uri)
            self.uri = uri # fp.get_uri ()
        except:
            self.uri = None
#        print "'%s'"%self.uri

        try:
            info = fp.query_info ("*")
            self.mediacontent = info.get_content_type ()
# self.mediacontent.find ('video') != 0 and 
            if self.mediacontent.find ('audio') != 0:
                self.uri = None
#            if self.mediacontent.find ('audio/midi') == 0: # midis segfaults sometimes
#                self.uri = None
        except:
            pass

        # create test pipeline
        src = gst.element_make_from_uri (gst.URI_SRC, uri, None)
        #src = gst.element_factory_make ('gnomevfssrc', 'gnomevfssrc')
        if src == None:
            return
        dec = gst.element_factory_make (self.decbin_name, 'decodebin')
        if dec == None:
            dec = gst.element_factory_make ('decodebin2', 'decodebin')
        if dec == None:
            dec = gst.element_factory_make ('decodebin', 'decodebin')
        if dec == None:
            return

        tfind = gst.element_factory_make ('typefind', 'tfind')
        if tfind == None:
            return

        self.pipeline = gst.Pipeline ('pipeline')
        self.pipeline.add (src, tfind, dec)
        gst.element_link_many (src, tfind, dec)

        dec.connect ("new-decoded-pad", self.newDecodedPadCb, None)
        dec.connect ("unknown-type", self.unknownTypeCb, None)
        tfind.connect ("have-type", self.haveTypeCb, None)

        # add watch
        bus = self.pipeline.get_bus ()
        bus.add_signal_watch ()
#        bus.enable_sync_message_emission ()
        bus.connect ("message", self.messageCb)

        self.track = 0
        self.item = MediaItem ()

    def run (self):
        if self.uri == None or self.pipeline == None:
            self.emit ("supported", False, None)
            return
        if self.mediacontent.find ("audio/prs.sid") == 0 :
#            dec = gst.element_factory_make ("siddec", None)
#            if dec == None:
#                self.emit ("supported", False, None)
#                return
            self.item.fullpath = self.uri
            self.item.mediacontent = self.mediacontent
            self.emit ("supported", True, self.item)
            return
        if self.mediacontent.find ("audio/midi") == 0 :
#            dec = gst.element_factory_make ("wildmidi", None)
#            if dec == None:
#                self.emit ("supported", False, None)
#                return
            self.item.fullpath = self.uri
            self.item.mediacontent = self.mediacontent
            self.emit ("supported", True, self.item)
            return

#        self.pipeline.get_by_name ('gnomevfssrc').set_property ("location", self.uri)
        self.pipeline.set_state (gst.STATE_PAUSED)
        glib.idle_add (self.getDurationIdleFunc, None)

    # FIXME: remove me. gio handles this part 
    def haveTypeCb (self, tfind, prob, caps, data):
        caps_str = caps.to_string ()

#        print caps_str
        if caps_str.find ('image') == 0 or caps_str.find ('text') == 0:
            self.emitted = True
            self.emit ("supported", False, None)

    def unknownTypeCb (self, demux, pad, caps, data):
        self.emitted = True
        self.emit ("supported", False, None)

    def newDecodedPadCb (self, demux, pad, last, data):
        fsink = gst.element_factory_make ("fakesink", None)
        self.sinks.append (fsink)
        fsink.set_property ("preroll-queue-len", 50)
        fsink.set_state (gst.STATE_PAUSED)

        self.pipeline.add (fsink)

        caps = pad.get_caps ()
        caps_str = caps.to_string ()

        spad = fsink.get_static_pad ('sink')
        pad.link (spad)

    def messageCb (self, bus, message):
        type = message.type

        if type == gst.MESSAGE_EOS:
            glib.idle_add (self.emitSuccessIdleFunc, self.item)

        elif type == gst.MESSAGE_ERROR:
            err, debug = message.parse_error ()
            # print "Error: %s" % err, debug
            self.emitted = True
            self.emit ("supported", False, None)
            self.pipeline.set_state (gst.STATE_NULL)
            state = self.pipeline.get_state ()

        elif type == gst.MESSAGE_STATE_CHANGED:
            if message.src != self.pipeline:
                return
            prev, cur, pending = message.parse_state_changed ()
            if prev == gst.STATE_READY and cur == gst.STATE_PAUSED:
                glib.idle_add (self.emitSuccessIdleFunc, self.item)

        elif type == gst.MESSAGE_DURATION:
            glib.idle_add (self.emitSuccessIdleFunc, self.item)

        elif type == gst.MESSAGE_TAG:
            taglist = message.parse_tag ()
            self.taglist = self.taglist.merge (taglist, gst.TAG_MERGE_APPEND)

    def parseTags (self, item):
        taglist = self.taglist

        try:
            item.album = taglist[gst.TAG_ALBUM]
        except:
            pass

        try:
            item.album_gain = taglist[gst.TAG_ALBUM_GAIN]
        except:
            pass

        try:
            item.album_peak = taglist[gst.TAG_ALBUM_PEAK]
        except:
            pass

        try:
            item.album_volume_count = taglist[gst.TAG_ALBUM_VOLUME_COUNT]
        except:
            pass

        try:
            item.album_volume_number = taglist[gst.TAG_ALBUM_VOLUME_NUMBER]
        except:
            pass

        try:
            item.artist = taglist[gst.TAG_ARTIST]
        except:
            pass

        try:
            item.audio_codec = taglist[gst.TAG_AUDIO_CODEC]
        except:
            pass

        try:
            item.bitrate = taglist[gst.TAG_BITRATE]
        except:
            pass

        try:
            item.codec = taglist[gst.TAG_CODEC]
        except:
            pass

        try:
            item.comment = taglist[gst.TAG_COMMENT]
        except:
            pass

        try:
            item.contact = taglist[gst.TAG_CONTACT]
        except:
            pass

        try:
            item.copyright = taglist[gst.TAG_COPYRIGHT]
        except:
            pass

        try:
            item.date = taglist[gst.TAG_DATE]
        except:
            pass

        try:
            item.description = taglist[gst.TAG_DESCRIPTION]
        except:
            pass

#        try:
#            item.duration = taglist[gst.TAG_DURATION]
#        except:
#            pass

        try:
            item.encoder = taglist[gst.TAG_ENCODER]
        except:
            pass

        try:
            item.encoder_version = taglist[gst.TAG_ENCODER_VERSION]
        except:
            pass

        try:
            item.extended_comment = taglist[gst.TAG_EXTENDED_COMMENT]
        except:
            pass

        try:
            item.genre = taglist[gst.TAG_GENRE]
        except:
            pass

        try:
            item.image = taglist[gst.TAG_IMAGE]
        except:
            pass

        try:
            item.isrc = taglist[gst.TAG_ISRC]
        except:
            pass

        try:
            item.language_code = taglist[gst.TAG_LANGUAGE_CODE]
        except:
            pass

        try:
            item.license = taglist[gst.TAG_LICENSE]
        except:
            pass

        try:
            item.location = taglist[gst.TAG_LOCATION]
        except:
            pass

        try:
            item.maximum_bitrate = taglist[gst.TAG_MAXIMUM_BITRATE]
        except:
            pass

        try:
            item.minimum_bitrate = taglist[gst.TAG_MINIMUM_BITRATE]
        except:
            pass

        try:
            item.nominal_bitrate = taglist[gst.TAG_NOMINAL_BITRATE]
        except:
            pass

        try:
            item.organization = taglist[gst.TAG_ORGANIZATION]
        except:
            pass

        try:
            item.performer = taglist[gst.TAG_PERFORMER]
        except:
            pass

        try:
            item.previev_image = taglist[gst.TAG_PREVIEW_IMAGE]
        except:
            pass

        try:
            item.serial = taglist[gst.TAG_SERIAL]
        except:
            pass

        try:
            item.title = taglist[gst.TAG_TITLE]
        except:
            pass

        try:
            item.track_count = taglist[gst.TAG_TRACK_COUNT]
        except:
            pass

        try:
            item.track_gain = taglist[gst.TAG_TRACK_GAIN]
        except:
            pass

        try:
            item.track_number = taglist[gst.TAG_TRACK_NUMBER]
        except:
            pass

        try:
            item.track_peak = taglist[gst.TAG_TRACK_PEAK]
        except:
            pass

        try:
            item.version = taglist[gst.TAG_VERSION]
        except:
            pass

        try:
            item.video_codec = taglist[gst.TAG_VIDEO_CODEC]
        except:
            pass

        # no need for list of tags
        if type (item.artist).__name__ == 'list':
            item.artist = item.artist[0]
        if type (item.title).__name__ == 'list':
            item.title = item.title[0]
        if type (item.album).__name__ == 'list':
            item.album = item.album[0]
        if type (item.track_number).__name__ == 'list':
            item.tarck_number = item.track_number[0]


    def getDurationIdleFunc (self, data):
        try:
            time = self.pipeline.query_duration (gst.Format (gst.FORMAT_TIME))[0]
            if time > 0:
                self.item.duration_i = time
                self.item.duration = convertGSTTimeToString (time)
        except:
            pass
        return False


    def emitSuccessIdleFunc (self, item):
        if self.emitted == True: # hack to stop videos emitted twice
            return False
        self.emitted = True

        item.fullpath = self.uri
        item.mediacontent = self.mediacontent
        self.parseTags (self.item)

        # HACK: try again if there were problems to get media duration
        if item.duration_i == 0:
            self.pipeline.set_state (gst.STATE_PAUSED)
            state = self.pipeline.get_state ()
            self.getDurationIdleFunc (None)

        # pass result to the treeview
        self.emit ("supported", True, self.item)

        self.pipeline.set_state (gst.STATE_NULL)
        state = self.pipeline.get_state ()
        return False
