import os
import gobject
import threading
import time

from database import Database
from mediaitem import MediaItem
from media import MediaTest

class MediaImporter (gobject.GObject, threading.Thread):
    __gsignals__ = { 'ready': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
                               (gobject.TYPE_UINT, )),
                     'add': (gobject.SIGNAL_RUN_FIRST, gobject.TYPE_NONE,
                                   (gobject.TYPE_PYOBJECT,))}
    lock = threading.Lock ()
    def __init__ (self, dbpath, dbfile):
        threading.Thread.__init__ (self)
        gobject.GObject.__init__ (self)

        self.file_count = 0
        self.ready = False
        self.force_quit = False
        self.filelist = []
        self.dbpath = dbpath
        self.dbfile = dbfile
        self.recursive = False

    def addFile (self, file):
        self.filelist.append (file)

    def addDirectory (self, dir):
        for file in os.listdir (dir):
            path = os.path.join (dir, file)
            if self.recursive == True and os.path.isdir (path) == True:
                self.addDirectory (path)
            else:
                self.addFile (path)

    def run (self):
        while self.force_quit == False:
            while len (self.filelist) == 0 and self.force_quit == False:
                time.sleep (0.001)
            if self.force_quit == False:
                self.filelist.sort ()
                self.realRun ()

    def forceQuit (self):
        self.force_quit = True

    def realRun (self):
        self.importList ()

        self.emit ("ready", 0)
        self.lock.acquire ()
        self.lock.release ()

    def importList (self):
        for fullpath in self.filelist:
            self.lock.acquire ()
            path, file = os.path.split (fullpath)
            if fullpath[0] == '/':
                fullpath = "file://" + fullpath

            test = MediaTest (fullpath)
            test.connect ("supported", self.supportedCb, file)
            test.start ()

        self.filelist = []

    def supportedCb (self, obj, supported, item, filename):

        try:
            obj.join ()
        except:
            pass

        if supported == False or item == None:
            self.lock.release ()
            return

        item.filename = filename

        self.db = Database (self.dbpath, self.dbfile)
        id = self.db.insertItem (item)
        item.id = id
        self.emit ("add", item)

        self.lock.release ()
