import pygtk
pygtk.require('2.0')
import gtk
import glib

import urllib

from database import *
from mediaitem import *

class MedialistTreeview(gtk.TreeView):
    __gsignals__ = { 'dnd-uri-add': ( gobject.SIGNAL_RUN_FIRST,
                                  gobject.TYPE_NONE,
                                  (gobject.TYPE_STRING,))}

    def __init__ (self, configdir, dbname):
        self.timestamp = 0
        self.tree_row_ref = None
        self.current_item = None

        self.db = Database (configdir, dbname)

        store = gtk.ListStore (gobject.TYPE_STRING, gobject.TYPE_STRING, 
                               gobject.TYPE_STRING, gobject.TYPE_UINT, gobject.TYPE_INT)

#        store.set_sort_func (4, self.sortMediaTreeviewModelCb)
#        store.set_default_sort_func (self.sortMediaTreeviewModelCb)
#        store.set_sort_column_id (-1, gtk.SORT_ASCENDING)

        gtk.TreeView.__init__ (self, store)

        treeselection = self.get_selection ()
        treeselection.set_mode (gtk.SELECTION_MULTIPLE)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn ('Artist - Title')
        column.set_resizable (True)
        self.append_column (column)
        column.pack_start (renderer, True)
        column.set_attributes (renderer, markup=0)

        renderer = gtk.CellRendererText ()
        column = gtk.TreeViewColumn('Lenght')
        column.set_resizable (True)
        self.append_column (column)
        column.pack_end (renderer, False)
        column.set_attributes (renderer, markup=1)

        self.set_headers_visible (True)

        dnd_list = [('treeview-row', gtk.TARGET_SAME_WIDGET, 0),
                    ('text/uri-list', 0, 1)]

        self.enable_model_drag_source (gtk.gdk.BUTTON1_MASK,
                                                          dnd_list,
                                                          gtk.gdk.ACTION_DEFAULT|gtk.gdk.ACTION_COPY)
        self.enable_model_drag_dest (dnd_list, gtk.gdk.ACTION_DEFAULT)

        self.connect ("drag-data-get", self.dragDataGetCb)
        self.connect ("drag-data-received", self.dragDataReceivedCb)

    def dragDataGetCb (self, treeview, context, selection, id, timestamp):
        treeselection = treeview.get_selection ()
        (model, paths) = treeselection.get_selected_rows ()

        iter = model.get_iter (paths[0])
        self.dropdata = model.get_value (iter, 3)
        data = model.get_value (iter, 2) # fullpath
        selection.set (selection.target, 8, data)

    def dragDataReceivedCb (self, treeview, context, x, y, selection, info, timestamp):
        if timestamp == self.timestamp:
            return

        self.timestamp = timestamp

        if selection == None or selection.data == None:
            return

        if info == 0:
            model = treeview.get_model ()

            id = self.dropdata
            item = self.db.getItemById (id)
            rowtitle = self.createRowTitleFromItem (item)
            ddata = [rowtitle, item.duration, selection.data, self.dropdata, -1]        
            rowpos = treeview.get_dest_row_at_pos (x, y)
            if rowpos:
                path, position = rowpos
                iter = model.get_iter (path)
 
                if position == gtk.TREE_VIEW_DROP_BEFORE or position == gtk.TREE_VIEW_DROP_INTO_OR_BEFORE:
                    niter = model.insert_before (iter, ddata)
                else:
                    niter = model.insert_after (iter, ddata)

            else:
                niter = model.append (ddata)

            if self.current_item != None and id == self.current_item.id:
                npath = model.get_path (niter)
                self.tree_row_ref = gtk.TreeRowReference (model, npath)
                self.setRowActive (model, niter, True)

            context.finish (True, True, timestamp)

        else:
            uris = selection.data.split ("\r\n")
            for urin in uris:
                if urin == '':
                    continue
                self.emit ("dnd-uri-add", urin)

    def sortMediaTreeviewModelCb (self, model, iter1, iter2):
        id = model.get_value (iter1, 3)
        item = self.db.getItemById (id)

        if item == None:
            return 1

        if item.artist == None:
            item.artist = ""
        if item.album == None:
            item.album = ""
        test1 = "%s%s-%03d-%s"% (item.artist, item.album, item.track_number, item.filename)

        # test2
        id = model.get_value (iter2, 3)
        item = self.db.getItemById (id)
        if item == None:
            return -1

        if item.artist == None:
            item.artist = ""
        if item.album == None:
            item.album = ""
        test2 = "%s%s-%03d-%s"% (item.artist, item.album, item.track_number, item.filename)

        print test1
        print test2

        return cmp (test1, test2)

    def addItemToList (self, item, id):
        # add item to the list
        select_first = False
        store = self.get_model ()

        if store.get_iter_first () == None:
            select_first = True

        artist_plus_title = self.createRowTitleFromItem (item)
        iter = store.append([artist_plus_title, item.duration, item.fullpath, id, -1])

        if select_first == True:
            treeselection = self.get_selection ()
            treeselection.select_iter (iter)
        return False

    def createRowTitleFromItem (self, item):
        # if there is no title or artist add file to the list
        if item.artist == '' or item.title == '':
            artist_plus_title = glib.markup_escape_text (item.filename)
        else:
            artist_plus_title = u''
            artist_plus_title += glib.markup_escape_text (item.artist) + " - "
            artist_plus_title += glib.markup_escape_text (item.title)
        return artist_plus_title

    def setRowActive (self, model, iter, active):
        id = model.get_value (iter, 3)
        item = self.db.getItemById (id)
        if item == None:
            return

        duration = ""
        artist_plus_title = glib.markup_escape_text (item.filename)

        if item.artist != "" and item.title != "":
            artist_plus_title = glib.markup_escape_text (item.artist)
            artist_plus_title += " - "
            artist_plus_title += glib.markup_escape_text (item.title)
 
        if item.duration != "":
            duration = item.duration

        if active == True:
            artist_plus_title = u'<b>%s</b>' % artist_plus_title 
            duration = u'<b>%s</b>' % duration

        model.set (iter, 0, artist_plus_title)
        model.set (iter, 1, duration)

    def setTreeRowRef (self, model, iter):
        path = model.get_path (iter)
        self.tree_row_ref = gtk.TreeRowReference (model, path)

    def clearTreeRowRef (self):
        self.tree_row_ref = None

    def getTreeRowRef (self):
        return self.tree_row_ref

    def setCurrentPath (self, path_str):
        if path_str == None:
            return
        model = self.get_model ()
        if model == None:
            return
        if len (model) == 0:
            return
#        iter = model.get_iter_from_string (path_str)
        path = (int (path_str),)
        iter = model.get_iter (path)
        if iter == None:
            iter = model.get_iter_first ()
        if iter == None:
            return

        path = model.get_path (iter)
        self.tree_row_ref = gtk.TreeRowReference (model, path)
        self.setRowActive (model, iter, True)

    def getCurrentPath (self):
        if self.tree_row_ref == None:
            return None
        model = self.tree_row_ref.get_model ()
        if model == None:
            return None
        path = self.tree_row_ref.get_path ()
        if path == None:
            return None
        iter = model.get_iter (path)
        return model.get_string_from_iter (iter)
