import gtk
import gobject

import os

class Preferences ():
    def __init__ (self):
        self.configpath = None # configure file path

        # window
        self.x = 100
        self.y = 100
        self.width = 400
        self.height = 300

        self.current_path = None # playing media

        # other
        self.volume = 1.0

        self.loadState ()

    def createConfigDir (self):
        home = os.getenv ("HOME")
        if os.path.isdir (home) == False:
            raise OSError ("'%s' is not directory." % home)
        configdir = os.path.join (home, ".config", "kilikaling")
        self.mkdir (configdir)
        return configdir

    # current path
    def getCurrentPath (self):
        return self.current_path
    def setCurrentPath (self, current_path):
        self.current_path = current_path

    # volume
    def getVolume (self):
        return self.volume
    def setVolume (self, volume):
        self.volume = volume

    # main window allocation
    def getWindowAllocation (self):
        return (self.x, self.y, self.width, self.height)
    def setWindowAllocation (self, x, y, width, height):
        self.x = x
        self.y = y
        self.width = width
        self.height = height

    def loadState (self):
        try:
            dir = self.createConfigDir ()
            self.configpath = os.path.join (dir, "kilikaling.cfg")

            # no config use defaults
            if os.path.isfile (self.configpath) == False:
                return

            fp = open (self.configpath, 'r')
            for line in fp:
                values = line.split ("=")
                if values[0] == "window.x":
                    self.x = int (values[1])
                elif values[0] == "window.y":
                    self.y = int (values[1])
                elif values[0] == "window.w":
                    self.width = int (values[1])
                elif values[0] == "window.h":
                    self.height = int (values[1])
                elif values[0] == "volume":
                    self.volume = float (values[1])
                elif values[0] == "current.path":
                    self.current_path = values[1]
                    fp.close ()
        except:
            pass

    def saveState (self):
        try:
            dir = self.createConfigDir ()
            self.configpath = os.path.join (dir, "kilikaling.cfg")

            fp = open (self.configpath, 'w')
            fp.write ("window.x=%d\n"%self.x)
            fp.write ("window.y=%d\n"%self.y)
            fp.write ("window.w=%d\n"%self.width)
            fp.write ("window.h=%d\n"%self.height)
            fp.write ("volume=%f\n"%self.volume)
            if self.current_path != None:
                fp.write ("current.path=%s\n"%self.current_path)
            fp.close ()

        except:
            pass

    def destroy (self):
        if self.dialog == None:
            return

        self.dialog.destroy ()

    def run (self, parent):
        self.dialog = gtk.Dialog ("Preferences...",
                                  parent,
                                  gtk.DIALOG_MODAL|gtk.DIALOG_DESTROY_WITH_PARENT,
                                  (gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT,
                                   gtk.STOCK_OK, gtk.RESPONSE_OK))

        vbox = dialog.get_content_area ()

        notebook = gtk.Notebook ()
        vbox.add (notebook)
        notebook.show ()

        # playback
        vbox = self.createNewPage (notebook, "Playback")

        # SID
        label = gtk.Label ("<b>SID</b>")
        label.set_use_markup (True)
        vbox.pack_start (label, False, False, 0)
        label.show ()

        # SID clock
        hbox, combo = self.createComboBoxLine ("Clock:")
        combo.append_text ("PAL")
        combo.append_text ("NTSC")
        combo.set_active (0)
        vbox.pack_start (hbox, False, False, 0)

        # SID memory
        hbox, combo = self.createComboBoxLine ("Memory:")
        combo.append_text ("Bank Switching")
        combo.append_text ("Transparent ROM")
        combo.append_text ("Playsid Environment")
        combo.set_active (0)
        vbox.pack_start (hbox, False, False, 0)

        check = self.createCheckButtonLine ("Measured volume")
        vbox.pack_start (check, False, False, 0)

        check = self.createCheckButtonLine ("MOS8580")
        vbox.pack_start (check, False, False, 0)

        check = self.createCheckButtonLine ("Force speed")
        vbox.pack_start (check, False, False, 0)

        self.dialog.run ()

        # TODO: collect stuff

    def createCheckButtonLine (self, title):
        check = gtk.CheckButton (title)
        check.show ()
        return check
        
    def createComboBoxLine (self, title):
        hbox = gtk.HBox (False, 0)
        label = gtk.Label (title)
        label.set_use_markup (True)
        hbox.pack_start (label, False, False, 0)
        label.show ()
        combo = gtk.combo_box_new_text ()
        hbox.pack_start (combo, False, False, 0)
        combo.show ()
        hbox.show ()
        return (hbox, combo)

    def createNewPage (self, notebook, title):
        label = gtk.Label ("<big><b>%s</b></big>"%title)
        label.set_use_markup (True)
        vbox = gtk.VBox (False, 0)
        notebook.append_page (vbox, label)
        vbox.show ()
        label.show ()
        return vbox

    def mkdir (self, dir):
        if os.path.isdir (dir):
            pass
        elif os.path.isfile (dir):
            raise OSError ("'%s' already exists." % newdir)
        else:
            head, tail = os.path.split (dir)
            if head and not os.path.isdir (head):
                self.mkdir (head)
            if tail:
                os.mkdir (dir)
